# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
#  file: line_functions.py
#  License: LICENSE.TXT
#  Author: Ioannis Tziakos
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
# -----------------------------------------------------------------------------
import re


# -----------------------------------------------------------------------------
#  Pre-compiled regexes
# -----------------------------------------------------------------------------
indent_regex = re.compile(r"\s+")


# -----------------------------------------------------------------------------
#  Constants
# -----------------------------------------------------------------------------
NEW_LINE = ""


# ------------------------------------------------------------------------------
#  Functions to manage indentation
# ------------------------------------------------------------------------------


def add_indent(lines, indent=4):
    """ Add spaces to indent a list of lines.

    Arguments
    ---------
    lines : list
        The list of strings to indent.

    indent : int
        The number of spaces to add.

    Returns
    -------
    lines : list
        The indented strings (lines).

    Notes
    -----
    Empty strings are not changed.

    """
    indent_str = " " * indent if indent != 0 else ""
    output = []
    for line in lines:
        if is_empty(line):
            output.append(line)
        else:
            output.append(indent_str + line)
    return output


def remove_indent(lines):
    """ Remove all indentation from the lines.

    Returns
    -------
    result : list
        A new list of left striped strings.

    """
    return [line.lstrip() for line in lines]


def trim_indent(lines):
    """ Trim global indentation level from lines.

    """
    non_empty_lines = filter(lambda x: not is_empty(x), lines)
    indent = set(len(get_indent(line)) for line in non_empty_lines)
    indent.discard(0)
    global_indent = min(indent)
    return [line[global_indent:] for line in lines]


def get_indent(line):
    """ Return the indent portion of the line.

    """
    indent = indent_regex.match(line)
    if indent is None:
        return ""
    else:
        return indent.group()


# ------------------------------------------------------------------------------
#  Functions to detect line type
# ------------------------------------------------------------------------------


def is_empty(line):
    return not line.strip()


# ------------------------------------------------------------------------------
#  Functions to adjust strings
# ------------------------------------------------------------------------------


def fix_star(word):
    """ Replace ``*`` with ``\\*`` so that it will be parsed properly by
    docutils.

    """
    return word.replace("*", "\\*")


def fix_backspace(word):
    """ Replace ``\\`` with ``\\\\`` so that it will printed properly in the
    documentation.

    """
    return word.replace("\\", "\\\\")


def fix_trailing_underscore(word):
    """ Replace the trailing ``_`` with ``\\_`` so that it will printed
    properly in the documentation.

    """
    if word.endswith("_"):
        word = word.replace("_", "\\_")
    return word


def replace_at(word, line, index):
    """ Replace the text in-line.

    The text in line is replaced (not inserted) with the word. The
    replacement starts at the provided index. The result is cliped to
    the input length

    Arguments
    ---------
    word : str
        The text to copy into the line.

    line : str
        The line where the copy takes place.

    index : int
        The index to start coping.

    Returns
    -------
    result : str
        line of text with the text replaced.

    """
    word_length = len(word)
    result = line[:index] + word + line[(index + word_length) :]
    return result[: len(line)]



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# interface_implementation.py - Example of implementing an interface

# --[Imports]-------------------------------------------------------------------
from __future__ import print_function
from traits.api import HasTraits, implements, Str, Instance
from interface_definition import IName


# --[Code]----------------------------------------------------------------------
class Person(HasTraits):
    implements(IName)

    first_name = Str("John")
    last_name = Str("Doe")

    # Implementation of the 'IName' interface:
    def get_name(self):
        """ Returns the name of an object. """
        return "%s %s" % (self.first_name, self.last_name)


# --[Example*]------------------------------------------------------------------
class Apartment(HasTraits):
    renter = Instance(IName)


william = Person(first_name="William", last_name="Adams")
apt1 = Apartment(renter=william)
print("Renter is: ", apt1.renter.get_name())
# Result: Renter is: William Adams



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# metadata.py --- Example of accessing trait metadata attributes

# --[Imports]-------------------------------------------------------------------
from __future__ import print_function
from traits.api import HasTraits, Int, List, Float, Instance, Any, TraitType


# --[Code]----------------------------------------------------------------------
class Foo(HasTraits):
    pass


class Test(HasTraits):
    i = Int(99)
    lf = List(Float)
    foo = Instance(Foo, ())
    any = Any([1, 2, 3])


# --[Example*]------------------------------------------------------------------
t = Test()

# Prints values of various metadata attributes for each of the traits.
print(t.trait("i").default)  # 99
print(t.trait("i").default_kind)  # value
print(t.trait("i").inner_traits)  # ()
print(t.trait("i").is_trait_type(Int))  # True
print(t.trait("i").is_trait_type(Float))  # False

print(t.trait("lf").default)  # []
print(t.trait("lf").default_kind)  # list
print(t.trait("lf").inner_traits)  # (<traits.traits.CTrait object at
#  0x01B24138>,)
print(t.trait("lf").is_trait_type(List))  # True
print(t.trait("lf").is_trait_type(TraitType))  # True
print(t.trait("lf").is_trait_type(Float))  # False
print(t.trait("lf").inner_traits[0].is_trait_type(Float))  # True

print(t.trait("foo").default)  # <undefined>
print(t.trait("foo").default_kind)  # factory
print(t.trait("foo").inner_traits)  # ()
print(t.trait("foo").is_trait_type(Instance))  # True
print(t.trait("foo").is_trait_type(List))  # False

print(t.trait("any").default)  # [1, 2, 3]
print(t.trait("any").default_kind)  # list
print(t.trait("any").inner_traits)  # ()
print(t.trait("any").is_trait_type(Any))  # True
print(t.trait("any").is_trait_type(List))  # False



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# keywords.py --- Example of trait keywords

# --[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Str


# --[Code]----------------------------------------------------------------------
class Person(HasTraits):
    # 'label' is used for Traits UI field labels;
    # 'desc' can be used for tooltips.
    first_name = Str("", desc="first or personal name", label="First Name")
    last_name = Str("", desc="last or family name", label="Last Name")



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.


# add_class_trait.py --- Example of mutually-referring classes
#                        using add_class_trait()


# --[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Instance


# --[Code]----------------------------------------------------------------------
# Defining mutually-referring classes using add_class_trait()
class Chicken(HasTraits):
    pass


class Egg(HasTraits):
    created_by = Instance(Chicken)


# Now that 'Egg' is defined, we can add the 'hatched_from' trait to
# solve the mutual-reference problem...

Chicken.add_class_trait("hatched_from", Instance(Egg))



# -*- coding: UTF-8 -*-
# ------------------------------------------------------------------------------
#  file: class_doc.py
#  License: LICENSE.TXT
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
# ------------------------------------------------------------------------------
from .base_doc import BaseDoc
from .line_functions import get_indent, replace_at, add_indent
from .definition_items import (
    MethodItem,
    AttributeItem,
    TableLineItem,
    max_attribute_length,
    max_attribute_index,
    ListItem,
)


class ClassDoc(BaseDoc):
    """ Docstring refactoring for classes.

    The class provides the following refactoring methods.

    Methods
    -------
    _refactor_attributes(self, header) :
        Refactor the attributes section to sphinx friendly format.

    _refactor_methods(self, header) :
        Refactor the methods section to sphinx friendly format.

    _refactor_as_items_list(self, header) :
        Refactor the Keywords section to sphinx friendly format.

    _refactor_notes(self, header) :
        Refactor the note section to use the rst ``.. note`` directive.

    _refactor_example(self, header) :
        Refactor the example section to sphinx friendly format.

    """

    def __init__(self, lines, headers=None):

        if headers is None:
            headers = {
                "Attributes": "attributes",
                "Methods": "methods",
                "Notes": "notes",
                "Keywords": "as_item_list",
                "Note": "notes",
                "Example": "example",
                "Examples": "example",
            }

        super(ClassDoc, self).__init__(lines, headers)
        return

    def _refactor_attributes(self, header):
        """Refactor the attributes section to sphinx friendly format"""
        items = self.extract_items(AttributeItem)
        lines = []
        for item in items:
            lines += item.to_rst()
        return lines

    def _refactor_methods(self, header):
        """Refactor the methods section to sphinx friendly format.

        """
        items = self.extract_items(MethodItem)
        lines = []
        if len(items) > 0:
            columns = self._get_column_lengths(items)
            border = "{0:=^{1}} {0:=^{2}}".format("", columns[0], columns[1])
            heading = "{0:<{2}} {1:<{3}}".format(
                "Method", "Description", columns[0], columns[1]
            )
            lines += [border]
            lines += [heading]
            lines += [border]
            for items in items:
                lines += items.to_rst(columns)
            lines += [border]
            lines += [""]
        lines = [line.rstrip() for line in lines]
        return lines

    def _refactor_notes(self, header):
        """Refactor the note section to use the rst ``.. note`` directive.

        """
        paragraph = self.get_next_paragraph()
        lines = [".. note::"]
        lines += add_indent(paragraph)
        return lines

    def _refactor_as_item_list(self, header):
        """ Refactor the a section to sphinx friendly item list.

        Arguments
        ---------
        header : str
            The header name that is used for the fields (i.e. ``:<header>:``).

        """
        items = self.extract_items(item_class=ListItem)
        lines = [":{0}:".format(header.lower()), ""]
        prefix = None if len(items) == 1 else "-"
        for item in items:
            lines += add_indent(item.to_rst(prefix))
        return lines

    def _refactor_example(self, header):
        """ Refactor the example section to sphinx friendly format.

        Arguments
        ---------
        header : str
            The header name that is used for the fields (i.e. ``:<header>:``).

        """
        paragraph = self.get_next_paragraph()
        lines = [".. rubric:: {0}".format(header), "", "::", ""]
        lines += add_indent(paragraph)
        return lines

    def _get_column_lengths(self, items):
        """ Helper function to estimate the column widths for the refactoring of
        the ``Methods`` section.

        The method finds the index of the item that has the largest function
        name (i.e. self.term) and the largest signature. If the indexes are not
        the same then checks to see which of the two items have the largest
        string sum (i.e. self.term + self.signature).

        """
        name_index = max_attribute_index(items, "term")
        signature_index = max_attribute_index(items, "signature")
        if signature_index != name_index:
            index = signature_index
            item1_width = len(items[index].term + items[index].signature)
            index = name_index
            item2_width = len(items[index].term + items[index].signature)
            first_column = max(item1_width, item2_width)
        else:
            index = name_index
            first_column = len(items[index].term + items[index].signature)

        first_column += 11  # Add boilerplate characters
        second_column = max_attribute_length(items, "definition")
        return (first_column, second_column)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# deferring_notification.py -- Example of notification with deferring

# --[Imports]-------------------------------------------------------------------
from __future__ import print_function
from traits.api import HasTraits, Instance, PrototypedFrom, Str


# --[Code]----------------------------------------------------------------------
class Parent(HasTraits):

    first_name = Str
    last_name = Str

    def _last_name_changed(self, new):
        print("Parent's last name changed to %s." % new)


class Child(HasTraits):

    father = Instance(Parent)
    first_name = Str
    last_name = PrototypedFrom("father")

    def _last_name_changed(self, new):
        print("Child's last name changed to %s." % new)


# --[Example*]------------------------------------------------------------------
dad = Parent(first_name="William", last_name="Chase")
# Output: Parent's last name changed to Chase.

son = Child(first_name="John", father=dad)
# Output: Child's last name changed to Chase.

# Change Parent's last_name
dad.last_name = "Jones"
# Output: Parent's last name changed to Jones.
#         Child's last name changed to Jones.

# Override Child's last_name
son.last_name = "Thomas"
# Output Child's last name changed to Thomas.

# Change Parent's last_name; Child's is not affected.
dad.last_name = "Riley"
# Output: Parent's last name changed to Riley.

# Reset Child's last_name
del son.last_name
# Output: Child's last name changed to Riley.

# Change to Parent now affects Child.
dad.last_name = "Simmons"
# Output: Parent's last name changed to Simmons.
#         Child's last name changed to Simmons.



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# interface_definition.py - Example of defining an interface

# --[Imports]-------------------------------------------------------------------
from traits.api import Interface


# --[Code]----------------------------------------------------------------------
class IName(Interface):
    def get_name(self):
        """ Returns a string which is the name of an object. """



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.


# adapt_metadata.py - Example of using 'adapt' metadata

# --[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Instance
from interface_definition import IName


# --[Code]----------------------------------------------------------------------
class Apartment(HasTraits):
    renter = Instance(IName, adapt="no")



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# dynamic_notification.py --- Example of dynamic notification

# --[Imports]-------------------------------------------------------------------
from __future__ import print_function
from traits.api import Float, HasTraits, Instance


# --[Code]----------------------------------------------------------------------
class Part(HasTraits):
    cost = Float(0.0)


class Widget(HasTraits):
    part1 = Instance(Part)
    part2 = Instance(Part)
    cost = Float(0.0)

    def __init__(self):
        self.part1 = Part()
        self.part2 = Part()
        self.part1.on_trait_change(self.update_cost, "cost")
        self.part2.on_trait_change(self.update_cost, "cost")

    def update_cost(self):
        self.cost = self.part1.cost + self.part2.cost


# --[Example*]------------------------------------------------------------------
w = Widget()
w.part1.cost = 2.25
w.part2.cost = 5.31
print(w.cost)
# Result: 7.56



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# wildcard_rules.py --- Example of trait attribute wildcard rules

# --[Imports]-------------------------------------------------------------------
from traits.api import Any, HasTraits, Int, Python


# --[Code]----------------------------------------------------------------------
class Person(HasTraits):
    temp_count = Int(-1)
    temp_ = Any
    _ = Python



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# scratch_adapter.py - Example of writing an adapter from scratch

# --[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, Instance, adapts
from interface_definition import IName
from interface_implementation import Person


# --[Code]----------------------------------------------------------------------
class PersonINameAdapter(HasTraits):

    # Declare what interfaces this adapter implements,
    # and for what class:
    adapts(Person, IName)

    # Declare the type of client it supports:
    client = Instance(Person)

    # Implement the adapter's constructor:
    def __init__(self, client):
        self.client = client

    # Implement the 'IName' interface on behalf of its client:
    def get_name(self):
        return "%s %s" % (self.client.first_name, self.client.last_name)



# ------------------------------------------------------------------------------
#  file: refactor_doc.py
#  License: LICENSE.TXT
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
# ------------------------------------------------------------------------------
from .function_doc import FunctionDoc
from .class_doc import ClassDoc

# ------------------------------------------------------------------------------
# Extension definition
# ------------------------------------------------------------------------------
def refactor_docstring(app, what, name, obj, options, lines):

    refactor = None
    if "class" in what:
        refactor = ClassDoc(lines)
    elif ("function" in what) or ("method" in what):
        refactor = FunctionDoc(lines)

    if refactor is not None:
        refactor.parse()


def setup(app):
    app.setup_extension("sphinx.ext.autodoc")
    app.connect("autodoc-process-docstring", refactor_docstring)



# -*- coding: utf-8 -*-
#
# Traits documentation build configuration file, created by
# sphinx-quickstart on Tue Jul 22 10:52:03 2008.
#
# This file is execfile()d with the current directory set to its containing
# dir.
#
# The contents of this file are pickled, so don't put values in the namespace
# that aren't pickleable (module imports are okay, they're removed
# automatically).
#
# All configuration values have a default value; values that are commented out
# serve to show the default value.
from __future__ import print_function
import datetime
import io
import os
import sys

import six


# The docset build will use slightly different formatting rules
BUILD_DOCSET = bool(os.environ.get("BUILD_DOCSET"))

# If your extensions are in another directory, add it here. If the directory
# is relative to the documentation root, use os.path.abspath to make it
# absolute, like shown here.
sys.path.append(os.path.abspath("_extensions"))
sys.path.append(os.path.abspath("../../"))


def mock_modules():
    """ Optionally Mock missing modules to allow autodoc based documentation.

    The ``traits.has_dynamics_view`` imports the traitsui module and
    thus traitsui is needed so that the ``autodoc`` extension can
    extract the docstrings from the has_dynamics_view module. This
    function optionally mocks the traitsui module so that the traits
    documentation can be built without the traitui optional dependency
    installed.

    .. note::

       The mock library is needed in order to mock the traitsui
       package.

    """

    MOCK_MODULES = []
    MOCK_TYPES = []

    # Check to see if we need to mock the traitsui package
    try:
        import traitsui
    except ImportError:
        # Modules that we need to mock
        MOCK_MODULES = [
            "traitsui",
            "traitsui.api",
            "traitsui.delegating_handler",
        ]

        # Collect the types from traitsui that are based on HasTraits
        # We will need to mock them in a special way so that
        # TraitDocumenter can properly identify and document traits.
        from traits.api import HasTraits, HasPrivateTraits

        MOCK_TYPES.append(
            ("traitsui.delegating_handler", "DelegatingHandler", (HasTraits,))
        )
        MOCK_TYPES.append(
            ("traitsui.view_element", "ViewSubElement", (HasPrivateTraits,))
        )
    else:
        return

    if six.PY2:
        try:
            from mock import MagicMock
        except ImportError:
            if len(MOCK_MODULES) != 0:
                print(
                    "NOTE: TraitsUI is not installed and mock is not "
                    "available to mock the missing modules, some classes "
                    "will not be documented"
                )
                return
    else:
        from unittest.mock import MagicMock

    # Create the custom types for the HasTraits based traitsui objects.
    TYPES = {
        mock_type: type(mock_type, bases, {"__module__": path})
        for path, mock_type, bases in MOCK_TYPES
    }

    class DocMock(MagicMock):
        """ The special sphinx friendly mocking class to mock missing packages.

        Based on the suggestion from http://docs.readthedocs.org/en/latest/faq.html#i-get-import-errors-on-libraries-that-depend-on-c-modules

        """

        @classmethod
        def __getattr__(self, name):
            if name in ("__file__", "__path__", "_mock_methods"):
                # sphinx does not like getting a Mock object in this case.
                return "/dev/null"
            else:
                # Return a mock or a custom type as requested.
                return TYPES.get(name, DocMock(mocked_name=name))

        # MagicMock does not define __call__ we do just to make sure
        # that we cover all cases.
        def __call__(self, *args, **kwards):
            return DocMock()

        @property
        def __name__(self):
            # Make sure that if sphinx asks for the name of a Mocked class
            # it gets a nice strings to use (instead of "DocMock")
            return self.mocked_name

    # Add the mocked modules to sys
    sys.modules.update(
        (mod_name, DocMock(mocked_name=mod_name)) for mod_name in MOCK_MODULES
    )

    # Report on what was mocked.
    print(
        "mocking modules {0} and types {1}".format(
            MOCK_MODULES, [mocked[1] for mocked in MOCK_TYPES]
        )
    )


mock_modules()

# General configuration
# ---------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    "refactordoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.autosummary",
    "traits.util.trait_documenter",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"

# General substitutions.
project = "traits"
copyright = "2008-{date.year}, Enthought Inc".format(date=datetime.date.today())

# The default replacements for |version| and |release|, also used in various
# other places throughout the built documents.
version_info = {}
traits_init_path = os.path.join("..", "..", "traits", "__init__.py")
with io.open(traits_init_path, "r", encoding="utf-8") as version_module:
    version_code = compile(version_module.read(), "__init__.py", "exec")
    exec(version_code, version_info)
version = release = version_info["__version__"]

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = "%B %d, %Y"

# List of documents that shouldn't be included in the build.
# unused_docs = []

# List of directories, relative to source directories, that shouldn't be
# searched for source files.
# exclude_dirs = []

# The reST default role (used for this markup: `text`) to use for all
# documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# Options for the autodoc extension.
autodoc_default_flags = ["members"]

autodoc_member_order = "bysource"


# Options for HTML output
# -----------------------

# Use enthought-sphinx-theme if available
try:
    import enthought_sphinx_theme

    html_theme_path = [enthought_sphinx_theme.theme_path]
    html_theme = "enthought"
except ImportError as exc:
    import warnings

    msg = """Can't find Enthought Sphinx Theme, using default.
                Exception was: {}
                Enthought Sphinx Theme can be downloaded from
                https://github.com/enthought/enthought-sphinx-theme"""
    warnings.warn(RuntimeWarning(msg.format(exc)))

    # Use old defaults if enthought-sphinx-theme not available

    # The name of an image file (within the static path) to place at the top
    # of the sidebar.
    html_logo = "e-logo-rev.png"

    # The name of an image file (within the static path) to use as favicon of
    # the docs.  This file should be a Windows icon file (.ico) being 16x16
    # or 32x32 pixels large.
    html_favicon = "et.ico"

    # The style sheet to use for HTML and HTML Help pages. A file of that name
    # must exist either in Sphinx' static/ path, or in one of the custom paths
    # given in html_static_path.
    html_style = "default.css"

# When using docset browsers like Dash and Zeal the side bar is redundant.
if BUILD_DOCSET:
    html_theme_options = {"nosidebar": "true"}

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = "Traits 5 User Manual"

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
html_last_updated_fmt = "%b %d, %Y"

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
html_use_modindex = BUILD_DOCSET

# If false, no index is generated.
html_use_index = BUILD_DOCSET

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, the reST sources are included in the HTML build as _sources/<name>.
# html_copy_source = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = "Traitsdoc"


# Options for LaTeX output
# ------------------------

# The paper size ('letter' or 'a4').
# latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
# latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author,
#  document class [howto/manual]).
latex_documents = [
    (
        "index",
        "Traits.tex",
        "Traits 5 User Manual",
        "Enthought, Inc.",
        "manual",
    )
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = "enthought_logo.jpg"
latex_logo = "e-logo-rev.png"

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# Additional stuff for the LaTeX preamble.
# latex_preamble = ''

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_use_modindex = True

# Options for Texinfo output
# --------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        "index",
        "traits",
        "Traits 5 User Manual",
        "Enthought, Inc.",
        "Traits",
        "Explicitly typed attributes for Python.",
        "Python",
    )
]



# -*- coding: UTF-8 -*-
# ------------------------------------------------------------------------------
#  file: function_doc.py
#  License: LICENSE.TXT
#  Author: Ioannis Tziakos
#
#  Copyright (c) 2011, Enthought, Inc.
#  All rights reserved.
# ------------------------------------------------------------------------------
from .base_doc import BaseDoc
from .line_functions import get_indent, add_indent
from .definition_items import ArgumentItem, ListItem


class FunctionDoc(BaseDoc):
    """Docstring refactoring for functions

    The class provides the following refactoring methods.

    Methods
    -------
    _refactor_arguments(self, header) :
        Refactor the Arguments and Parameters section to sphinx friendly format.

    _refactor_as_items_list(self, header) :
        Refactor the Returns, Raises and Yields sections to sphinx friendly
        format.

    _refactor_notes(self, header) :
        Refactor the note section to use the rst ``.. note`` directive.

    """

    def __init__(self, lines, headers=None):

        if headers is None:
            headers = {
                "Returns": "as_item_list",
                "Arguments": "arguments",
                "Parameters": "arguments",
                "Raises": "as_item_list",
                "Yields": "as_item_list",
                "Notes": "notes",
                "Note": "notes",
            }

        super(FunctionDoc, self).__init__(lines, headers)
        return

    def _refactor_as_item_list(self, header):
        """ Refactor the a section to sphinx friendly item list.

        Arguments
        ---------
        header : str
            The header name that is used for the fields (i.e. ``:<header>:``).

        """
        items = self.extract_items(item_class=ListItem)
        lines = [":{0}:".format(header.lower())]
        if len(items) > 0:
            prefix = None if len(items) == 1 else "-"
            for item in items:
                lines += add_indent(item.to_rst(prefix))
        else:
            paragraph = self.get_next_paragraph()
            lines += add_indent(paragraph)

        return lines

    def _refactor_arguments(self, header):
        """ Refactor the argument section to sphinx friendly format.

        Arguments
        ---------
        header : unused
            This parameter is ingnored in thi method.

        """
        items = self.extract_items(item_class=ArgumentItem)
        lines = []
        for item in items:
            lines += item.to_rst()
        return lines

    def _refactor_notes(self, header):
        """ Refactor the notes section to sphinx friendly format.

        Arguments
        ---------
        header : unused
            This parameter is ingnored in this method.

        """
        paragraph = self.get_next_paragraph()
        lines = [".. note::"]
        lines += add_indent(paragraph)
        return lines



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# --(Adaptation)----------------------------------------------------------------
"""
Adaptation
==========

*Adaptation* is the process of transforming an object that does not implement
a specific interface, or set of interfaces, into one that does.

Defining Adapters
-----------------

In traits, an *adapter* is a special type of class whose role is to *transform*
some type of object which does not implement a specific interface, or set of
interfaces, into one that does.

Traits provides several different ways of writing adapters. We'll begin with
the simplest way, which is to create a subclass of **Adapter**.

Subclassing Adapter
-------------------

The **Adapter** base class is designed specifically for creating adapter
classes. It is actually a very simple class which streamlines the process of
creating new adapter classes by:

- Providing a standard constructor which normally does not need to be
  overridden by subclasses.

- Only requiring use of the **adapts** function to define the adapter.

The standard constructor for the **Adapter** class has the form::

    adapter_subclass_name(object_to_be_adapted)

where *adapter_subclass_name* is the name of the **Adapter** subclass.

The only thing the constructor does is::

    self.adaptee = object_to_be_adapted

which assigns the object being adapted to the *adaptee* trait.

As an adapter writer, the only things you need to add to the subclass
definition are:

- An **adapts** function call declaring which interfaces the adapter class
  implements on behalf of the object it is adapting.

  The form of the **adapts** function is as follows::

    adapts(client_class, interface [, interface2, ..., interfacen])

- A declaration for the *adaptee* trait (usually as an **Instance** of a
  particular class).

- The actual implementations of the interfaces declared in the **adapts**
  call. Usually the implementation code will be written in terms of the
  adapter class's *adaptee* trait assigned by the class constructor.

The following shows the definition of a simple adapter class::

  from traits.api import Adapter, Instance, implements

  class PersonINameAdapter (Adapter):

      # Declare what interfaces this adapter implements for its client:
      adapts(Person, IName)

      # Declare the type of client it supports:
      adaptee = Instance(Person)

      # Implement the 'IName' interface on behalf of its client:
      def get_name (self):
          return ('%s %s' % (self.adaptee.first_name, self.adaptee.last_name))

Rolling You Own Adapter Classes
-------------------------------

Note that using the **Adapter** base class is simply a convenience. It is not
necessary for an adapter class to derive from **Adapter**. However, if you do
not derive your adapter class from **Adapter**, then it is your responsibility
to provide all of the same information and setup implicitly provided by
**Adapter**.

In particular, in addition to using the *adapts* function to declare the set of
interfaces the class implements for its client object, you must also define the
constructor, or whatever means you define for binding the object to be adapted
to the adapter.

Creating an adapter class from scratch, we can re-write the previous adapter
example as follows::

  from traits.api import HasTraits, Instance, adapts

  class PersonINameAdapter(HasTraits):

      # Declare what interfaces this adapter implements, and for who:
      adapts(Person, IName)

      # Declare the type of client it supports:
      client = Instance(Person)

      # Implement the adapter's constructor:
      def __init__ (self, client):
          self.client = client

      # Implement the 'IName' interface on behalf of its client:
      def get_name (self):
          return ('%s %s' % (self.client.first_name, self.client.last_name))

As you can see, the main difference between this example and the last is:

- Explicit implementation of the adapter constructor.

Yet Another Way To Define Adapters
----------------------------------

It is also possible to declare a class to be an adapter class external to the
class definition itself, as shown in the following example::

    class AnotherPersonAdapter (object):

        # Implement the adapter's constructor:
        def __init__ (self, person):
            self.person = person

        # Implement the 'IName' interface on behalf of its client:
        def get_name (self):
            return ('%s %s' % (self.person.first_name, self.person.last_name))

    ...

    adapts(AnotherPersonAdapter, Person, IName)

When used in this way, the form of the **adapts** function is::

    adapts(adapter_class, client_class, interface [, interface2, ...,
           interfacen])

This form simply inserts the adapter class as the first argument (when
**adapts** is used inside of a class definition, this information is implicitly
available from the class itself).

Using Adapters
--------------

Now for the good part... how do you use adapters?

And the answer is... you don't. At least not explicitly.

In traits, adapters are created automatically whenever you assign an object to
an *interface* **AdaptsTo** or **AdaptedTo** trait and the object being
assigned does not implement the required interface. In this case, if an
adapter class exists that can adapt the specified object to the required
interface, an instance of the adapter class will be created for the object,
and the resulting adapter object is what ends up being assigned to the trait,
along with the original object. When using the **AdaptedTo** trait, the
adapter is assigned as the value of the trait, and the original object is
assigned as its *mapped* value. For the **AdaptsTo** trait, the original
object is assigned as the trait value, and the adapter is assigned as its
*mapped* value. In the case where the object does not require an adapter, the
object and adapted value are the same.

Note that it might happen that no adapter class exists that will adapt the
object to the required interface, but a pair, or series, of adapter classes
exist that will together adapt the object to the needed interface. In this
case, the required set of adapters will automatically be created for the
object and the final link in the chain adapter object (the one that actually
implements the required interface for some object class) will be used.

Whenever a situation like this arises, the adapted object used will always
contain the smallest set of available adapter objects needed to adapt the
original object.

The following code shows a simple example of using adaptation::

    # Create a Person object (which does not implement the 'IName' interface):
    william = Person(first_name = 'William', last_name = 'Adams')

    # Create an apartment, and assign 'renter' the previous object. Since
    # the value of 'renter' must implement 'IName', a 'PersonINameAdapter'
    # object is automatically created and assigned:
    apt = Apartment(renter = william)

    # Verify that the resulting value implements 'IName' correctly:
    print('Renter is: ', apt.renter.get_name())

    # Check the type of object actually assigned to 'renter':
    print(apt.renter)

Refer to the **Output** tab for the actual result of running this example.

Controlling Adaptation
----------------------

The **AdaptedTo** and **AdaptsTo** traits are actually subclasses of the
**Instance** trait. Normally, adaptation occurs automatically when values are
assigned to an **AdaptedTo** or **AdaptsTo** trait. However, any of the
**Instance**, **AdaptedTo** and **AdaptsTo** traits allow you to control how
adaptation is performed by means of the *adapt* metadata, which can have one of
the following values:

no
    Adaptation is not allowed (This is the default for the **Instance** trait).

yes
    Adaptation is allowed. If adaptation fails, an exception is raised (This is
    the default for both the **AdaptedTo** and **AdaptsTo** traits).

default
    Adapation is allowed. If adaptation fails, the default value for the trait
    is assigned instead.

As an example of modifying the adaptation behavior of an **AdaptedTo** trait,
we could rewrite the example **Apartment** class as follows::

    class Apartment(HasTraits):

        renter = AdaptedTo(IName, adapt = 'no')

Using this definition, any value assigned to *renter* must itself implement
the **IName** interface, otherwise an exception is raised. Try modifying and
re-running the example code to verify that this is indeed the case.
"""

# --<Imports>-------------------------------------------------------------------

from __future__ import print_function
from traits.api import *


# --[IName Interface]-----------------------------------------------------------
# Define the 'IName' interface:
class IName(Interface):
    def get_name(self):
        """ Returns the name of an object. """


# --[Person Class]--------------------------------------------------------------
class Person(HasTraits):

    first_name = Str("John")
    last_name = Str("Doe")


# --[PersonINameAdapter Class]--------------------------------------------------
class PersonINameAdapter(Adapter):

    # Declare what interfaces this adapter implements for its client:
    adapts(Person, IName)

    # Declare the type of client it supports:
    adaptee = Instance(Person)

    # Implementation of the 'IName' interface on behalf of its client:
    def get_name(self):
        """ Returns the name of an object. """
        return "%s %s" % (self.adaptee.first_name, self.adaptee.last_name)


# --[Apartment Class]-----------------------------------------------------------
# Define a class using an object that implements the 'IName' interface:
class Apartment(HasTraits):

    renter = AdaptedTo(IName)


# --[Example*]------------------------------------------------------------------
# Create an object implementing the 'IName' interface:
william = Person(first_name="William", last_name="Adams")

# Create an apartment, and assign 'renter' an object implementing 'IName':
apt = Apartment(renter=william)

# Verify that the object works correctly:
print("Renter is:", apt.renter.get_name())

# Check the type of object actually assigned to 'renter':
print(apt.renter)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# external_adapter.py - Example of declaring a class as an adapter
#                       externally to the class

# --[Imports]-------------------------------------------------------------------
from traits.api import adapts
from interface_definition import IName
from interface_implementation import Person


# --[Code]----------------------------------------------------------------------
class AnotherPersonAdapter(object):

    # Implement the adapter's constructor:
    def __init__(self, person):
        self.person = person

    # Implement the 'IName' interface on behalf of its client:
    def get_name(self):
        return "%s %s" % (self.person.first_name, self.person.last_name)


adapts(AnotherPersonAdapter, Person, IName)



# THIS FILE IS GENERATED FROM TRAITS SETUP.PY
version = '5.1.0'
full_version = '5.1.0'
git_revision = 'bc896728d443c8417f175c799353b2e55b63833f'
is_released = True

if not is_released:
    version = full_version



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# transient_metadata.py - Example of using 'transient' metadata

# --[Imports]-------------------------------------------------------------------
from traits.api import HasTraits, File, Any


# --[Code]----------------------------------------------------------------------
class DataBase(HasTraits):

    # The name of the data base file:
    file_name = File

    # The open file handle used to access the data base:
    file = Any(transient=True)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# wizard.py ---Example of a traits-based wizard UI
from traits.api import HasTraits, Int, Str


class Person(HasTraits):
    name = Str
    age = Int
    street = Str
    city = Str
    state = Str
    pcode = Str


bill = Person()
bill.configure_traits(kind="modal")



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# trait_reuse.py --- Example of reusing trait definitions

from traits.api import HasTraits, Range

# --[Code]----------------------------------------------------------------------
coefficient = Range(-1.0, 1.0, 0.0)


class quadratic(HasTraits):
    c2 = coefficient
    c1 = coefficient
    c0 = coefficient
    x = Range(-100.0, 100.0, 0.0)



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.

# minimal.py --- Minimal example of using traits.

# --[Imports]-------------------------------------------------------------------
from __future__ import print_function
from traits.api import HasTraits, Float, TraitError


# --[Code]----------------------------------------------------------------------
class Person(HasTraits):
    weight = Float(150.0)


# --[Example*]------------------------------------------------------------------
# instantiate the class
joe = Person()

# Show the default value
print(joe.weight)

# Assign new values
joe.weight = 161.9  # OK to assign a float
print(joe.weight)

joe.weight = 162  # OK to assign an int
print(joe.weight)

# The following line causes a traceback:
try:
    joe.weight = "average"  # Error to assign a string
    print("You should not see this message.")
except TraitError:
    print("You can't assign a string to the 'weight' trait.")



#  Copyright (c) 2007, Enthought, Inc.
#  License: BSD Style.


# all_wildcard.py --- Example of trait attribute wildcard rules

# --[Imports]-------------------------------------------------------------------
from __future__ import print_function
from traits.api import Any, Str, Int, HasTraits, TraitError


# --[Code]----------------------------------------------------------------------
class Person(HasTraits):

    # Normal, explicitly defined trait:
    name = Str

    # By default, let all traits have any value:
    _ = Any

    # Except for this one, which must be an Int:
    age = Int


# --[Example*]------------------------------------------------------------------
# Create a sample Person:
bill = Person()

# These assignments should all work:
bill.name = "William"
bill.address = "121 Drury Lane"
bill.zip_code = 55212
bill.age = 49

# This should generate an error (must be an Int):
print("Attempting to assign a string to an Int trait object...\n")
try:
    bill.age = "middle age"
except TraitError as c:
    print("TraitError: ", c, "\n")

# Display the final results:
bill.print_traits()
